﻿# -*- coding: utf-8 -*-
import ConfigParser

from Modules.read_ini import read_parametrs
data_ini = read_parametrs()

from panda3d.core import loadPrcFileData
loadPrcFileData('', 'fullscreen '+str(data_ini.fullscreen))
loadPrcFileData('', 'sync-video '+str(data_ini.sync_video))

import direct.directbase.DirectStart
from pandac.PandaModules import *
from direct.showbase.DirectObject import DirectObject
from direct.gui.DirectGui import *
from direct.actor.Actor import Actor
import sys

class Start(DirectObject): # класс
    def __init__(self):
        
        global data_ini
        self.props = WindowProperties()

        alight = AmbientLight('alight')
        alight.setColor(VBase4(0.1, 0.1, 0.1, 1))
        alnp = render.attachNewNode(alight)
        render.setLight(alnp)
    
        light = Spotlight('plight')
        light.setColor(VBase4(1, 1, 1, 1))
        self.plnp = base.camera.attachNewNode(light)
        self.plnp.node().setAttenuation( Vec3( 0, 0.05, 0 ))
        self.plnp.node().getLens().setFov(150)
        self.plnp.node().setExponent(128.0)
        render.setLight(self.plnp)
        render.setShaderAuto()

        # HUD
        myFrame = DirectFrame(frameColor=(1, 0, 0, 0.5), frameSize = (-0.002, 0.002, -0.002, 0.002),image=('Data/Gui/Cur.png'), scale = (0.07,0,0.07))
        myFrame.setTransparency(1) 
        myFrame.setColor(1,1,1,0.4)
        
        base.disableMouse() # отключаем управление по умолчанию
        base.camLens.setFov(75)
        base.camLens.setNear(0.01)
        
        self.Golova = 3 # Переменная, устанавливает камеру и уровень головы
        
        self.x = 0 # Движение влево и вправо
        self.y = 0 # Движение вперед и назад
        self.z = 0 # Движение вверх и вниз
        
        self.keyMap = {"Forward":0, "Back":0, "Left":0, "Right":0} # Переменная с ключем "Forward,Back,Left" со значением "0"

        self.accept("w", self.setKey, ["Forward",1]) # клавиша нажата "w", передадим функции "setKey" ключ "Forward" со значением "1"
        self.accept("w-up", self.setKey, ["Forward",0]) # клавиша отпущена "w-up", передадим функции "setKey" ключ "Forward" со значением "0"
        self.accept("s", self.setKey, ["Back",1])
        self.accept("s-up", self.setKey, ["Back",0])
        self.accept("a", self.setKey, ["Left",1])
        self.accept("a-up", self.setKey, ["Left",0])
        self.accept("d", self.setKey, ["Right",1])
        self.accept("d-up", self.setKey, ["Right",0])
        self.accept("enter", self.Shader_Bump)
        
        self.accept('escape', self.GUI_show_hide) # если была нажата Esc то запускаем функцию меню 'GUI_show_hide'

        # Физика #
        self.world = OdeWorld() # Создадим мир ODE с названием "world"
        self.world.setGravity(0, 0, -9.81) # Установим силу гравитации по вектору z в мире "world"
        self.world.initSurfaceTable(1)
        self.world.setSurfaceEntry(0, 0, 150, 0.0, 9.1, 1, 0.00001, 100, 0.002)
        # Создание контакных групп
        self.space = OdeSimpleSpace()
        self.space.setAutoCollideWorld(self.world)
        self.contactgroup = OdeJointGroup()
        self.space.setAutoCollideJointGroup(self.contactgroup)

        # Игрок #
        self.player_mod =Actor('Data/Objects/player_mod', {'walk' : 'Data/Objects/panda-walk.egg'})
        #self.player_mod.loop("walk")
        self.player_mod.setH(180) # Поворачивем как нам надо
        self.player_mod.setZ(0) # Устанавливаем высоту
        
        self.player = NodePath("Player") # Создание нода для модели игрока с переменной 
        self.player_mod.reparentTo(self.player) # Прицепляем к ноду модель
        self.player.setScale(0.3) # Маштабируем
        self.player.setPos(0, 0, 1.1) # Координаты расположения игрока
        #self.player.reparentTo(render) # Вносим в рендер

        # Создание физической модели
        self.boxBody = OdeBody(self.world)
        M = OdeMass()
        M.setBox(180, 0.5, 0.5, 1)
        self.boxBody.setMass(M)
        self.boxBody.setPosition(self.player.getPos(render)) # Устанавлеваем позицию физического бокса как у модели
        # Создание пораметров геометрии
        boxGeom = OdeSphereGeom(self.space, 0.9)
        boxGeom.setCollideBits(BitMask32(0x00000001))
        boxGeom.setCategoryBits(BitMask32(0x00000001))
        boxGeom.setBody(self.boxBody)
        
        self.game = '' # состояние игры
        self.statys = '' # состояние меню
        self.sum = 0 # переменная для подсчета загруженных моделей
        self.Node_list = [] # сюда записываем имена нодов
        self.file = 'off' # состояние был ли прочитан файл уровня
        self.name_level = '' # имя уровня который надо загрузить
        self.index_mod = 0 # индекс модели
        
        self.fon = DirectFrame(image=('Data/Gui/fon_menu.png')) # фрейм для картинки для фона
        self.fon.setScale(base.getAspectRatio(), 1, 1) # масштабируем !возможно у кого HD экран может придется коректировать.

        self.ops = DirectFrame(image=('Data/Gui/fon_ops.png')) # фрейм для картинки настроек
        self.ops.setScale(base.getAspectRatio(), 1, 1)
        self.ops.hide() # Скрываем фон для настроек

        # здесь кнопки меню
        self.Start = DirectButton(pos=(0, 0, -0.4), scale=.17,command=self.clear_level_run, extraArgs = ['level_1'], image = ('Data/Gui/Start.png'), image_scale = (base.getAspectRatio(), 0, 0.5), borderWidth = (0,0), relief=0)
        self._b_ops = DirectButton(pos=(0, 0, -0.6), scale=.17,command=self.options_show, image = ('Data/Gui/ops.png'), image_scale = (base.getAspectRatio(), 0, 0.5), borderWidth = (0,0), relief=0)
        self.Exit = DirectButton(pos=(0, 0, -0.8), scale=.17,command=sys.exit, image = ('Data/Gui/exit.png'), image_scale = (base.getAspectRatio(), 0, 0.5), borderWidth = (0,0), relief=0)
        
        # здесь кнопки меню опций
        self._b_ops_enter = DirectButton(pos=(0, 0, -0.6), scale=.17,command=self.save_ini_config, image = ('Data/Gui/enter.png'), image_scale = (base.getAspectRatio(), 0, 0.5), borderWidth = (0,0), relief=0)
        self._b_ops_enter.hide() # скрываем кнопку ввода в меню опциях
        
        self._b_ops_exit = DirectButton(pos=(0, 0, -0.8), scale=.17,command=self.options_hide, image = ('Data/Gui/menu.png'), image_scale = (base.getAspectRatio(), 0, 0.5), borderWidth = (0,0), relief=0)
        self._b_ops_exit.hide() # скрываем кнопку выхода в меню опциях
        
        self.DCB_fullscreen = DirectCheckButton(text = "-fullscreen-", indicatorValue=int(data_ini.fullscreen), scale=.05, boxPlacement = 'right', relief=1, boxRelief=2, boxBorder=0.2)
        self.DCB_fullscreen.hide() # скрываем кнопку выбора режима окна в меню опциях
        
        self.DCB_sync_video = DirectCheckButton(pos=(0, 0, -0.1),text = "-sync_video-", indicatorValue=int(data_ini.sync_video), scale=.05, boxPlacement = 'right', relief=1, boxRelief=2, boxBorder=0.2)
        self.DCB_sync_video.hide() # скрываем кнопку выбора режима синхронизации в меню опциях
        
        # менеджер который который запускает функцию загрузки
        taskMgr.add(self.Load, 'Load')

        self.bar = DirectWaitBar(pos=(0,0,-0.7)) # полоска загрузки, парметры
        self.bar.hide() # скрываем полоску при иницилизации
        self.bar.setScale(0.5,1,0.6) # масштабируем по осям
        
        self.shaderenable = 1

            
    def Load(self, task): # Функция загрузки уровней(файла)
        if (self.name_level != ''):
            if (self.file == 'off'):
            
                self.Exit.hide() # скрываем кнопку выход
                self.Start.hide() # скрываем кнопку карт
                self._b_ops.hide() # скрываем кнопку опции
                
                self.fon['image'] = ('Data/Gui/fon_load.png') # ставим на фон картинку загрузки
                self.bar.show() # отображаем полоску загрузки
                try: 
                    Map = open('Levels/'+str(self.name_level)+'.lev', "rb") # читаем файл
                finally:
                    self.list_Ob = Map.readlines() # создаем из строк файла список
                    Map.close() # закрываем файл
                    self.sum = len(self.list_Ob) # подсчитываем количество строк(моделей)
                    self.bar['range'] = self.sum # присваиваем полоске загрузки максимальное значение на оснавании количества строк(т.е. файлов)
                    self.file = 'on' # ставим статус что файл причитан и не нужно выполнять эту часть кода
                    
            model = self.list_Ob[self.index_mod] # получаем строку по индексу     0               1              2
            data = model.split() # создаем список из строки, наглядный пример: 'teapot' 'Data/Objects/teapot' '0,0,0'
                
            name_mod = data[0] # берем имя
            path_mod = data[1] # путь модели
            coord = data[2] # берем координаты из списка                                           x   y   z
            pos = coord.split(',') # теперь создаём список координат из строки, наглядный пример: '0' '0' '0'
            
            object = loader.loadModel(path_mod) # создаём переменную для загрузчика со значением пути взятым из списка по индексу 1
            object.flattenLight() # Сбрасываем транформацию
            # Создание пораметров геометрии
            groundTrimesh = OdeTriMeshData(object, False)
            groundGeom = OdeTriMeshGeom(self.space, groundTrimesh)
            groundGeom.setCategoryBits(BitMask32(0x00000001))
            
            ModelNode = render.attachNewNode(str(name_mod)) # создаём нод для модели в рендере со значением имени из списка по индексу 0
            ModelNode.setPos(float(pos[0]), float(pos[1]), float(pos[2])) # устанавливаем координаты

            ModelNode.setTag('DirModel' , path_mod) # устанавливаем тег, со значением пути от куда была загруженна модель
            object.reparentTo(ModelNode) # цепляем модель к ноду
            self.Node_list.append(str(name_mod))# вносим имя нода в список
                
            self.bar['value'] += 1 # с каждой загруженной моделью прибавляем значение полоске
            self.index_mod += 1 # с каждой загруженной моделью прибавляем индекс
            
            if (self.sum == self.bar['value']): # проверяем не загружены ли все модели, сравниваем количество строк со значением полоски
                self.bar.hide() # если достигнуто то скрываем полоску загрузки
                self.fon.hide() # за одно и фрейм фона
                self.fon['image'] = ('Data/Gui/fon_menu.png') # ставим изображение для меню
                self.bar['value'] = 0 # значения полоски загрузки, сброс
                self.sum = 0 # переменная для подсчета загруженных моделей, сброс
                self.game = 'run' # ставим значение что игра запущена
                self.statys = 'on' # ставим статус для меню, что меню доступно при нажатии Esc
                self.name_level = '' # сбрасываем имя загруженного уровня
                self.file = 'off' # сбрасываем  статус файла
                self.index_mod = 0 # сбрасываем индекс модели
                self.props.setCursorHidden(True)
                base.win.requestProperties(self.props)
                # Запускаем физику и управление
                taskMgr.add(self.simulationTask, 'Physics Simulation')
                
                
        if (self.statys == 'on'):
        
            self.boxBody.addRelForce(self.x, self.y, self.z)
            self.accept("space", self.boxBody.setForce, [0 , 0 , 6000])
        
            dt=globalClock.getDt()
        
            if (self.keyMap["Forward"]!=0):  
                self.y = 800
            else:
                if (self.keyMap["Back"]!=0):
                    self.y = -800
                else:
                    self.y = 0

            if (self.keyMap["Left"]!=0):
                self.x = -800
            else:
                if (self.keyMap["Right"]!=0):
                    self.x = 800
                else:
                    self.x = 0
        
            # Обновление позиции камеры
            VerPos = self.player.getX()
            GorPos = self.player.getY()
            Visota = self.player.getZ()+self.Golova
            base.camera.setPos(VerPos, GorPos, Visota,)
  
            md = base.win.getPointer(0)
            x = md.getX()
            y = md.getY()
            if base.win.movePointer(0, base.win.getXSize()/2, base.win.getYSize()/2):
                base.camera.setH(base.camera.getH() -  (x - base.win.getXSize()/2)*0.1)
                base.camera.setP(base.camera.getP() - (y - base.win.getYSize()/2)*0.1)
                self.player.setH(base.camera.getH())
                if (base.camera.getP()<=-89.01):
                    base.camera.setP(-89)
                if (base.camera.getP()>=89.01):
                    base.camera.setP(89)
                
        return  task.cont # возвращаем менеджеру задание на запуск этой функции.
        
    def Shader_Bump(self):
        if (self.shaderenable):
            self.shaderenable = 0
            for md in self.Node_list: # проходим по списку для извлечения имен нодов
                node = render.find(md) # ищем в рендере нод
                node.setShaderOff()
        else:
            self.shaderenable = 1
            for md in self.Node_list: # проходим по списку для извлечения имен нодов
                node = render.find(md) # ищем в рендере нод
                node.setShaderAuto()

    def GUI_show_hide(self): # функция для скрытия и отображения меню
        if (self.game == 'run'): # проверка была ли запущена игра, а то глупо как то мигать меню по Esc, с начала.
            if (self.statys == 'on'): # теперь проверяем последний статус меню
                taskMgr.remove('Physics Simulation')
                self.props.setCursorHidden(False) # Отображаем курсор
                base.win.requestProperties(self.props) # Устанавливем системе
                
                self.fon.show() # отображаем фрейм фона
                self.Exit.show() # отображаем кнопку выход
                self.Start.show() # отображаем кнопку старт
                self._b_ops.show() # отображаем кнопку опции
                
                self.statys = 'off' # меняем статус
                
            else: # если открыто т.е. 'off'
                taskMgr.add(self.simulationTask, 'Physics Simulation')
                self.props.setCursorHidden(True) # Скрываем курсор
                base.win.requestProperties(self.props) # Устанавливем системе
                
                self.fon.hide() # скрываем фрейм фона
                self.Exit.hide() # скрываем кнопку выход
                self.Start.hide() # скрываем кнопку старт
                self._b_ops.hide() # скрываем кнопку опции
                
                self.statys = 'on' # меняем статус
                
    def options_show(self): # функция для отображения опций
        self.fon.hide() # скрываем фрейм фона
        self.Exit.hide() # скрываем кнопку выход
        self.Start.hide() # скрываем кнопку старт
        self._b_ops.hide() # скрываем кнопку опции
        
        self.ops.show() # отображаем фон опции
        self._b_ops_enter.show() # отображаем кнопку ввода в меню опциях
        self._b_ops_exit.show() # отображаем кнопку выхода из опции
        
        self.DCB_fullscreen.show() # отображаем кнопку выбора режима окна в меню опциях
        self.DCB_sync_video.show() # отображаем кнопку выбора режима синхронизации в меню опциях
        
    def options_hide(self): # функция для скрытия опций
        self.ops.hide() # скрываем фон опции
        self._b_ops_enter.hide() # скрываем кнопку ввода в меню опциях
        self._b_ops_exit.hide() # скрываем кнопку выхода из опции
        
        self.DCB_fullscreen.hide() # скрываем кнопку выбора режима окна в меню опциях
        self.DCB_sync_video.hide() # скрываем кнопку выбора режима синхронизации в меню опциях
        
        self.fon.show() # отображаем фрейм фона
        self.Exit.show() # отображаем кнопку выход
        self.Start.show() # отображаем кнопку старт
        self._b_ops.show() # отображаем кнопку опции
        
    def save_ini_config(self): # функция для сохранения в файл ini заначений
        config = ConfigParser.RawConfigParser() # создаём объект необходимый для работы с Config фалов
        config.read('Config.ini') # читаем указаный файл
        config.set('render', 'fullscreen', self.DCB_fullscreen['indicatorValue']) # запишем переменной 'full_mode' значение
        config.set('render', 'sync_video', self.DCB_sync_video['indicatorValue']) # запишем переменной 'sync' значение
        with open('Config.ini', 'wb') as configfile: # записываем в файл
            config.write(configfile)

    def setKey(self, key, value): # Функция для перезаписи в переменную "keyMap" ключа и значения
        self.keyMap[key] = value

    def simulationTask(self, task): # функция симуляции
        dt=globalClock.getDt()
        if dt>0.05:
           dt=0.05
        self.world.quickStep(dt)
        self.contactgroup.empty()
        self.space.autoCollide()
        self.player.setPos(self.boxBody.getPosition())
        self.boxBody.setQuaternion(self.player.getQuat())
        self.boxBody.setAngularVel(0, 0, 0)
        return task.cont
        
    def nam_lev(self, name): # функция для присвоения переменной имени уровня который нужно загрузить
        self.name_level = name
        
    def clear_level_run(self, name): # функция очистки. Для загрузки следующего уровня надо очистить сцену. 
        if (self.game == 'run'): # проверка был ли запуск игры(загрузка), глупо делать чистку если ещё игра не была загружена.
            self.game = '' # состояние игры, сброс
            self.statys = '' # состояние меню, сброс
            for Ob in self.Node_list: # проходим по списку для извлечения имен нодов
                node = render.find(Ob) # ищем в рендере нод
                node.detachNode() # отсоеденяем
                node.removeNode() # удаляем
                loader.unloadModel(node.getTag('DirModel')) # по идеи должен выгружать из памяти модели, но они всё-таки остаются.
            del self.Node_list[:] # чистим лист с именами моделей
            camera.setPos(0) # сбрасываем координаты камеры
            camera.setHpr(0) # сбрасываем углы вращения камеры
            self.nam_lev(name) # после этого устанавливаем имя нового уровня и загружаем
        else:
            self.nam_lev(name) # если не было ещё загрузки, устанавливае имя уровня и загружаем
            
Start() # запускаем класс
run()

